@@ -79,8 +79,7 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         self.m_lessonfile = lessonfile
         self.parse_lessonfile()
         if self.m_P and self.m_statistics:
-            solfege.db.validate_stored_statistics(
-                self.m_P.header.lesson_id, self.m_P.m_filename)
+            solfege.db.validate_stored_statistics(self.m_P.m_filename)
     def parse_lessonfile(self):
         self.m_question = None
         self.q_status = self.QSTATUS_NO
@@ -518,13 +517,13 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     def do_test_complete(self):
         self.on_end_practise()
         req = self.m_t.m_P.get_test_requirement()
-        res = self.m_t.m_statistics.get_last_test_result()
+        self.g_cancel_test.hide()
+        solfege.win.exit_test_mode()
+        passed, res = solfege.db.get_test_status(self.m_t.m_P.m_filename)
         if res >= req:
             gu.dialog_ok(_("Test completed!\nYour score was %(score).1f%%.\nThe test requirement was %(requirement).1f%%.") % {'score': res * 100, 'requirement': req * 100})
         else:
             gu.dialog_ok(_("Test failed.\nYour score was %(score).1f%%.\nThe test requirement was %(requirement).1f%%.") % {'score': res * 100, 'requirement': req * 100})
-        self.g_cancel_test.hide()
-        solfege.win.exit_test_mode()
     def set_lesson_heading(self, txt):
         if txt:
             self.g_lesson_heading.set_text('<span size="large"><b>%s</b></span>' % gu.escape(txt))
@@ -558,7 +557,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
                     self.m_key_bindings[s]()
                     return 1
     def keymatch(self, event, cfname):
-        a, b = utils.parse_key_string(self.get_string(cfname))
+        a, b = gu.parse_key_string(self.get_string(cfname))
         return ((event.state & (gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK|gtk.gdk.MOD1_MASK)) == a) and (event.keyval == b)
     def setup_statisticsviewer(self, viewclass, heading):
         self.g_statview = viewclass(self.m_t.m_statistics, heading)
@@ -619,10 +618,10 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         if 'm_mpd_varname' in dir(exception):
             m.add_text(_('Failed to parse the music in the variable "%(varname)s" in question number %(idx)i in the lesson file "%(lf)s".') % {
                 'idx': idx,
-                'lf': self.m_t.m_P.m_filename, 
+                'lf': lessonfile.uri_expand(self.m_t.m_P.m_filename),
                 'varname': exception.m_mpd_varname})
         else:
-            m.add_text(_('Failed to parse the music for question number %(idx)i in the lesson file "%(lf)s".') % {'idx': idx, 'lf': self.m_t.m_P.m_filename})
+            m.add_text(_('Failed to parse the music for question number %(idx)i in the lesson file "%(lf)s".') % {'idx': idx, 'lf': lessonfile.uri_expand(self.m_t.m_P.m_filename)})
         if 'm_mpd_badcode' in dir(exception):
             m.add_nonwrapped_text(exception.m_mpd_badcode)
         m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
