@@ -130,12 +130,11 @@ def check_rcfile():
 
 
 class SolfegeApp(cfg.ConfigUtils):
-    def __init__(self, options, lessonfile_manager):
+    def __init__(self, options):
         """
         options -- command line options parsed by optparse
         """
         cfg.ConfigUtils.__init__(self, 'solfege-app')
-        self.m_startup_complete = False
         lessonfile.MusicBaseClass.temp_dir = tempfile.mkdtemp(prefix="solfege-")
         os.environ['SOLFEGETEMPDIR'] = lessonfile.MusicBaseClass.temp_dir
         # test_mode is when we are running a test from the Tests menu
@@ -143,7 +142,6 @@ class SolfegeApp(cfg.ConfigUtils):
         self.m_options = options
         self.m_teachers = {}
         self.m_running_exercise = None
-        self.lessonfile_manager = lessonfile_manager
         self.m_sound_init_exception = None
         #
         self.m_userman_language = "C"
@@ -151,60 +149,6 @@ class SolfegeApp(cfg.ConfigUtils):
             if os.path.isdir(os.path.join('help', lang)):
                 self.m_userman_language = lang
                 break
-        # We have to check if lesson files have changed since last time we practised.
-        self._pp = lessonfile_manager.parse(options.debug if options else None)
-        gobject.idle_add(self.reparse_lessonfiles)
-    def nonidle_reparse(self):
-        total = len(self.lessonfile_manager.m_uiddb) * 2
-        count = 0.0
-        for uuid, data in self.lessonfile_manager.parse(self.m_options.debug):
-            yield count / total
-            count += 1
-        from solfege import lessonfilegui
-        lessonfilegui.handle_lesson_id_crash(lessonfile.manager)
-        for uuid in self.lessonfile_manager.iterate_lesson_ids():
-            data = lessonfile.manager.m_uiddb[uuid]
-            solfege.db.set_lessonfile_data(uuid, data['header']['title'],
-                    data['header']['module'], data['filename'])
-            yield count / total
-            count += 1
-    def reparse_lessonfiles(self):
-        """
-        Parse all lesson files when we are idle.
-        """
-        # Check that solfege.win has been created. It is possible that
-        # it is not created the first time we are idle, if the user is
-        # prompted a question during MainWin construction
-        if not getattr(solfege, 'win', None):
-            return True
-        try:
-            uuid, data = self._pp.next()
-            return True
-        except StopIteration:
-            solfege.win.handle_lesson_id_crash()
-            self._pp = lessonfile.manager.iterate_lesson_ids()
-            gobject.idle_add(self.check_for_changed_lessonfiles)
-            return False
-    def check_for_changed_lessonfiles(self):
-        """
-        Feed the parsed lesson file data to the database. We do
-        it when we are idle.
-        """
-        try:
-            uuid = self._pp.next()
-            data = lessonfile.manager.m_uiddb[uuid]
-            solfege.db.set_lessonfile_data(uuid, data['header']['title'],
-                    data['header']['module'], data['filename'])
-            return True
-        except StopIteration:
-            del self._pp
-        self.m_startup_complete = True
-        # If the user is really fast, or the computer not very idle, the
-        # page displayed might need to be updated. It can contain warning
-        # signs instead of links to some exercises.
-        if solfege.win.m_viewer in ('frontpage', 'frontpage'):
-            solfege.win.get_view().display_data(solfege.win.get_view()._page_tree)
-        return False
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
@@ -311,48 +255,38 @@ class SolfegeApp(cfg.ConfigUtils):
     def show_exercise_theory(self):
         if self.m_teachers[self.m_running_exercise].m_P.header.theory:
             solfege.win.display_docfile("%s.html" % self.m_teachers[self.m_running_exercise].m_P.header.theory, "")
-    def _practise_lesson_id(self, lesson_id, urlobj=None):
+    def _practise_lessonfile(self, filename, urlobj=None):
         """
         return the module name.
         """
-        module = solfege.db.get_field(lesson_id, 'module')
+        module = lessonfile.infocache.get(filename, 'module')
         if self.m_running_exercise:
             solfege.win.box_dict[self.m_running_exercise].on_end_practise()
         if module not in self.m_teachers:
             self.create_teacher(module)
         if module not in solfege.win.box_dict:
             solfege.win.initialise_exercise(self.m_teachers[module])
-        self.m_teachers[module].set_lessonfile(
-            solfege.db.get_field(lesson_id, 'filename'))
+        self.m_teachers[module].set_lessonfile(filename)
         if self.m_teachers[module].m_P:
             solfege.win.activate_exercise(module, urlobj)
         self.m_running_exercise = module
         self.m_teachers[module].g_view = solfege.win.box_dict[module]
         solfege.win.show_help_on_current()
         return module
-    def practise_lesson_id(self, lesson_id):
+    def practise_lessonfile(self, filename):
         def cleanup():
-            module = solfege.db.get_field(lesson_id, 'module')
+            module = lessonfile.infocache.get(filename, 'module')
             self.m_teachers[module].m_P = None
             solfege.win.box_dict[module].practise_box.set_sensitive(False)
             solfege.win.box_dict[module].config_box.set_sensitive(False)
             solfege.win.box_dict[module].action_area.set_sensitive(False)
             solfege.win.box_dict[module].std_buttons_end_practise()
         try:
-            module = self._practise_lesson_id(lesson_id)
+            module = self._practise_lessonfile(filename)
         except (lessonfile.LessonfileParseException,
                 dataparser.DataparserException), e:
             cleanup()
-            solfege.win.display_exception_message(
-                e, lessonfile=solfege.db.get_field(lesson_id, 'filename'))
-            return
-        # The try-except clause above will not catch trying to parse a
-        # lesson_id that does not exist. This because
-        # abstract.Teacher.parse_lessonfile will catch the exception and
-        # display a message informing the user. So we have to check m_P here.
-        # FIXME it can look as there is room for some redesign here...
-        if not solfege.win.box_dict[module].m_t.m_P:
-            cleanup()
+            solfege.win.display_exception_message(e, lessonfile=filename)
             return
         if 'm_discards' in dir(self.m_teachers[module].m_P):
             for msg in self.m_teachers[module].m_P.m_discards:
@@ -365,9 +299,9 @@ class SolfegeApp(cfg.ConfigUtils):
         if w:
             w.set_sensitive(bool(self.m_teachers[module].m_P.header.theory))
         return module
-    def test_lesson_id(self, lesson_id):
+    def test_lessonfile(self, filename):
         self.m_test_mode = True
-        module = self.practise_lesson_id(lesson_id)
+        module = self.practise_lessonfile(filename)
         solfege.win.box_dict[module].on_start_practise()
         solfege.win.enter_test_mode()
     def handle_href(self, href, display_docfile_set_adj=1):
@@ -423,7 +357,7 @@ class SolfegeApp(cfg.ConfigUtils):
         # the Url class does not know anything about lesson ids, but we
         # assume that lesson_ids don't include the '/' character.
         if '/' not in urlobj.lessonfile:
-            self.practise_lesson_id(urlobj.lessonfile)
+            self.practise_lesson_id(urlobj.lessonfile)#FIXME practise_lessonfile
             return
         if urlobj.lessonfile:
             if not os.path.isfile(urlobj.lessonfile):
@@ -431,7 +365,7 @@ class SolfegeApp(cfg.ConfigUtils):
                 return
         p = lessonfile.LessonfileCommon()
         p.parse_file(urlobj.lessonfile)
-        self._practise_lesson_id(p.header.lesson_id, urlobj)
+        self._practise_lessonfile(p.header.lesson_id, urlobj)#FIXME test
         # This loop changes variables in the lesson file header if they
         # have set values in the url.
         for key, val in urlobj.config.iteritems():
@@ -494,12 +428,12 @@ class SolfegeApp(cfg.ConfigUtils):
         table = reportlib.Table()
         report.append(table)
         for lesson_info in export_data:
-            lesson_id = lesson_info['lesson_id']
-            module = solfege.db.get_field(lesson_id, 'module')
+            filename = lesson_info['filename']
+            module = lessonfile.infocache.get(filename, 'module')
             if module not in self.m_teachers:
                 self.create_teacher(module)
             p = self.m_teachers[module].lessonfileclass()
-            p.parse_file(solfege.db.get_field(lesson_id, 'filename'))
+            p.parse_file(lessonfile.uri_expand(filename))
             for c in range(lesson_info['count']):
                 if module == 'idbyname':
                     p.select_random_question()
@@ -520,7 +454,7 @@ class SolfegeApp(cfg.ConfigUtils):
                     soundcard.end_export()
                 elif module in ('melodicinterval', 'harmonicinterval'):
                     t = self.m_teachers[module]
-                    t.set_lessonfile(solfege.db.get_field(lesson_id, 'filename'))
+                    t.set_lessonfile(filename)
                     t.start_practise()
                     t.new_question("c", "c''")
                     t.q_status = t.QSTATUS_SOLVED
@@ -601,11 +535,11 @@ class SolfegeApp(cfg.ConfigUtils):
                  and not sdict['count'] because sometimes the app has some
                  questions, and just need a few more.
         """
-        module = solfege.db.get_field(sdict['lesson_id'], 'module')
+        module = lessonfile.infocache.get(sdict['filename'], 'module')
         if module not in self.m_teachers:
             self.create_teacher(module)
         p = self.m_teachers[module].lessonfileclass()
-        p.parse_file(solfege.db.get_field(sdict['lesson_id'], 'filename'))
+        p.parse_file(lessonfile.uri_expand(sdict['filename']))
         if module == 'idbyname':
             for x in self._sheet_gen_question_idbyname(p, count, sdict):
                 yield x
@@ -642,11 +576,10 @@ class SolfegeApp(cfg.ConfigUtils):
                 yield ret
     def _sheet_gen_question_interval(self, module, p, count, sdict):
         # FIXME in the idbyname we count how many times each question
-        # has been selected, so get an even selection. We don't do it
+        # has been selected, to get an even selection. We don't do it
         # here at the moment, because we need to descide what we really want.
         teacher = self.m_teachers[module]
-        teacher.set_lessonfile(solfege.db.get_field(sdict['lesson_id'],
-                                                          'filename'))
+        teacher.set_lessonfile(sdict['filename'])
         teacher.start_practise()
         for x in range(count):
             teacher.new_question("c'", "c''")
@@ -688,36 +621,4 @@ class SolfegeApp(cfg.ConfigUtils):
                     teacher.m_tonika.get_octave_notename(),
                     (teacher.m_tonika + teacher.m_interval).get_octave_notename())
                 yield ret
-    def load_learning_tree_as_frontpage_data(self, filename):
-        """
-        This function can read the old "learning tree" format datafile and
-        convert it to a front page structure.
-        Load into the .m_frontpage_data variable
-        """
-        data = {}
-        data['collections'] = {}
-        data['columns'] = [[]]
 
-        d = eval(open(filename, "r").read())
-        for k in d['menu']:
-            data['columns'][0].append({'heading': k['name'].replace("_", ""), 'collection-ids': []})
-            for k2 in k['children']:
-                newkey = "%s-%s" % (k['name'].lower().replace(" ", "_"), k2['name'].lower().replace(" ", "_"))
-                assert newkey not in data
-                data['collections'][newkey] = k2
-                data['columns'][0][-1]['collection-ids'].append(newkey)
-                del k2['nametotranslate']
-                k2['name'] = k2['name'].replace("_", "")
-
-        lines = sum(len(x['collection-ids'])+1 for x in data['columns'][0])
-        count = 0
-        for idx, x in enumerate(data['columns'][0]):
-            count += len(x['collection-ids']) + 1
-            if count > lines / 2:
-                if len(data['columns']) == 1:
-                    data['columns'].append([])
-                    for i in range(idx+1, len(data['columns'][0])):
-                        data['columns'][1].append(data['columns'][0][idx+1])
-                        del data['columns'][0][idx+1]
-                    break
-        self.m_frontpage_data = data
