@@ -38,13 +38,12 @@ import solfege
 
 if sys.platform == 'win32':
     try:
-        from soundcard import winmidi
+        from solfege.soundcard import winmidi
     except ImportError, e:
-        print "Loading winmidi.pyd failed:"
-        print e
+        print >> sys.stderr, "Loading winmidi.pyd failed:"
+        print >> sys.stderr, e
         winmidi = None
 
-
 class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
     def on_destroy(self, widget, e):
         solfege.win.g_config_window.destroy()
@@ -77,7 +76,8 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             gtk.Adjustment(self.get_int('config/default_bpm'), 10, 500, 1, 10))
         self.g_arpeggio_bpm = gu.nSpinButton('config', 'arpeggio_bpm',
             gtk.Adjustment(self.get_int('config/arpeggio_bpm'), 10, 500, 1, 10))
-        for text, widget in [("_Default:",self.g_default_bpm), ("A_rpeggio:",self.g_arpeggio_bpm)]:
+        for text, widget in [(_("_Default:") ,self.g_default_bpm), 
+                             (_("A_rpeggio:") ,self.g_arpeggio_bpm)]:
             label = gtk.Label(text)
             label.set_alignment(0.0, 0.5)
             label.set_use_underline(True)
@@ -429,7 +429,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
              'label': _("MIDI:"),
              'players': [
                          ('gst-launch', 'playbin uri=file://%s',
-                                        'filesrc location=lesson-files/share/fanfare.midi ! wildmidi ! alsasink'),
+                                        'filesrc location=exercises/lesson-files/share/fanfare.midi ! wildmidi ! alsasink'),
                          ('timidity', '-idqq %s'),
                          ('drvmidi', ''),
                          ('playmidi', ''),
@@ -501,7 +501,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             category_vbox.pack_start(box)
     def test_XXX_player(self, w, typeid, testfile):
         try:
-            soundcard.play_mediafile(typeid, os.path.join('lesson-files/share', testfile))
+            soundcard.play_mediafile(typeid, os.path.join('exercises/lesson-files/share', testfile))
         except osutils.BinaryBaseException, e:
             solfege.win.display_error_message2(e.msg1, e.msg2)
     def create_linux_sound_page(self):
