@@ -48,6 +48,30 @@ def escape(s):
     s = s.replace("<", "&lt;")
     return s.replace(">", "&gt;")
 
+def get_modifier(s):
+    m = (('<ctrl>', gtk.gdk.CONTROL_MASK),
+         ('<shift>', gtk.gdk.SHIFT_MASK),
+         ('<alt>', gtk.gdk.MOD1_MASK))
+    for mod, mask in m:
+        if s.startswith(mod):
+            return mask, s[len(mod):]
+    return None, s
+
+def parse_key_string(string):
+    if not string:
+        return None, None
+    mod = 0
+    m, s = get_modifier(string)
+    while m:
+        mod = mod + m
+        m, s = get_modifier(s)
+    if len(s) == 1:
+        return mod, ord(s)
+    else:
+        return mod, getattr(gtk.keysyms, s)
+
+
+
 def tLabel(table, x1, x2, y1, y2, text="", xalign=0.0, yalign=0.5, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL, xpadding=0, ypadding=0):
     label = gtk.Label(text)
     label.set_alignment(xalign, yalign)
@@ -549,15 +573,9 @@ class EditorDialogBase(object):
     # their file name as key.
     instance_dict = {}
     def __init__(self, filename=None):
+        self.m_instance_number = EditorDialogBase.instance_counter
+        EditorDialogBase.instance_counter += 1
         self.m_filename = filename
-        if filename:
-            assert filename not in self.instance_dict
-            self.instance_dict[filename] = self
-        else:
-            self.m_instance_number = EditorDialogBase.instance_counter
-            EditorDialogBase.instance_counter += 1
-            assert (None, self.m_instance_number) not in self.instance_dict
-            self.instance_dict[(None, self.m_instance_number)] = self
         self.m_savetime = time.time()
         self.connect('delete_event', self.on_delete_event)
         self.g_ui_manager = gtk.UIManager()
@@ -573,6 +591,13 @@ class EditorDialogBase(object):
          ('Open', gtk.STOCK_OPEN, None, None, None, self.on_open),
          ('Help', gtk.STOCK_HELP, None, None, None, self.on_show_help),
         ])
+    def add_to_instance_dict(self):
+        if self.m_filename:
+            assert self.m_filename not in self.instance_dict
+            self.instance_dict[self.m_filename] = self
+        else:
+            assert (None, self.m_instance_number) not in self.instance_dict
+            self.instance_dict[(None, self.m_instance_number)] = self
     def get_idict_key(self):
         """
         Return the key used to store the editor in .instance_dict.
@@ -673,14 +698,12 @@ class EditorDialogBase(object):
         if ret == gtk.RESPONSE_OK:
             try:
                 new_file = decode_filename(dialog.get_filename())
-                if (not self.m_filename) and (not self.m_changed):
-                    del self.instance_dict[self.get_idict_key()]
                 assert new_file not in self.instance_dict
                 win = self.__class__(new_file)
                 win.show_all()
-                self.instance_dict[new_file] = self
                 win.set_title(new_file)
                 if (not self.m_filename) and (not self.m_changed):
+                    del self.instance_dict[self.get_idict_key()]
                     self.destroy()
             except Exception, e:
                 # Since we catch all sorts of exceptions here, we don't have
@@ -688,7 +711,6 @@ class EditorDialogBase(object):
                 # PractiseSheet.parse_file. Just let int() raise ValueException
                 # if the string in not an integer.
                 solfege.win.display_exception_message(e)
-                win.destroy()
         dialog.destroy()
     def on_save_as(self, widget):
         """
@@ -742,8 +764,12 @@ class EditorDialogBase(object):
         if not self.m_filename:
             retval = self.on_save_as(widget)
         else:
-            self.save()
-            retval = True
+            try:
+                self.save()
+                retval = True
+            except IOError, e:
+                dialog_ok(_("Error saving file"), self, str(e), gtk.MESSAGE_ERROR)
+                retval = False
         if retval:
             self.m_savetime = time.time()
         return retval
@@ -824,6 +850,15 @@ class LogWindow(gtk.Window):
 
 
 class ClickableLabel(gtk.LinkButton):
-    def __init__(self, *w):
-        gtk.LinkButton.__init__(self, *w)
+    def __init__(self, label):
+        """
+        We are abusing the Link button by not giving it a real uri.
+        So let ut not even pretend to follow its api.
+        """
+        gtk.LinkButton.__init__(self, label, label)
         self.get_children()[0].set_alignment(0.0, 0.5)
+    def make_warning(self):
+        im = gtk.Image()
+        im.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
+        self.set_image(im)
+        self.set_alignment(0.0, 0.5)
