@@ -172,9 +172,12 @@ class PianoOctaveWithAccelName(PianoKeyboard):
     def __init__(self, callback, keys):
         PianoKeyboard.__init__(self, 1, "c", 40)
         self.m_callback = callback
+        self.m_visible_accels = False
         self.m_keys = keys
     def draw(self, context):
         PianoKeyboard.draw(self, context)
+        if not self.m_visible_accels:
+            return
         context.save()
         context.select_font_face("Sans")
         context.set_font_size(24)
