@@ -1,4 +1,5 @@
 # GNU Solfege - free ear training software
+
 # vim: set fileencoding=utf-8 :
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Tom Cato Amundsen
 #
@@ -19,6 +20,7 @@ from __future__ import absolute_import
 
 import glob
 import locale
+import logging
 import os
 import random
 import re
@@ -28,6 +30,8 @@ import sys
 import textwrap
 import time
 
+import gobject
+
 from solfege import cfg
 from solfege import dataparser
 from solfege import filesystem
@@ -43,6 +47,40 @@ from solfege.mpd.rat import Rat
 
 _test_mode = False
 
+# The name of the folder containing the standard learning trees.
+# Relative to the installation dir.
+exercises_dir = u"exercises"
+_abs_exercises_dir = os.path.join(os.getcwdu(), exercises_dir) + os.sep
+
+solfege_uri = u"solfege:"
+
+def uri_expand(url):
+    """
+    Convert from solfege: URI to normal file name.
+    """
+    assert isinstance(url, unicode)
+    if url.startswith(solfege_uri):
+        url = os.path.join(exercises_dir, url[len(solfege_uri):])
+    return url
+
+def mk_uri(filename):
+    """
+    Make an URI
+    solfege:path/to/file
+    that refers to files in exercises/ if the file is below that directory.
+    Return unchanged if not.
+    """
+    if os.path.isabs(filename):
+        if filename.startswith(_abs_exercises_dir):
+            return (u"solfege:%s" % filename[len(_abs_exercises_dir):]).replace(os.sep, "/")
+    else:
+        if filename.startswith(u"%s%s" % (exercises_dir, os.sep)):
+            return (u"solfege:%s" % filename[len("%s%s" % (exercises_dir, os.sep)):]).replace(os.sep, "/")
+    return filename
+
+def is_uri(filename):
+    return filename.startswith(solfege_uri)
+
 class LessonfileException(Exception):
     pass
 
@@ -774,7 +812,7 @@ class Cmdline(MusicBaseClass):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         try:
             osutils.PopenSingleton(str(self.m_musicdata).split(" "),
-                   cwd='lesson-files')
+                   cwd=lessonfile_ref.m_location)
         except OSError, e:
             raise osutils.RunningExecutableFailed(self.m_musicdata)
 
@@ -860,9 +898,9 @@ class LessonfileCommon(object):
         self.header     a Header instance
         self.questions  a list of all question
         """
-        self.m_location = os.path.split(filename)[0]
+        self.m_location = os.path.split(uri_expand(filename))[0]
         self.m_filename = filename
-        self.parse_string(open(filename, 'rU').read(), really_filename=filename)
+        self.parse_string(open(uri_expand(filename), 'rU').read(), really_filename=filename)
     def parse_string(self, s, really_filename=None):
         """
         See parse_file docstring.
@@ -1201,6 +1239,12 @@ class TestSupport(object):
         q = range(len(self.m_questions)) * count
         random.shuffle(q)
         return q
+    def get_test_num_questions(self):
+        """
+        Return the number of questions in the running test.
+        This method is undefined if not running a test.
+        """
+        return len(self.m_questions) * parse_test_def(self.header.test)[0]
     def get_test_requirement(self):
         """
         Return the amount of exercises that has to be correct to
@@ -1345,6 +1389,12 @@ class IntervalsLessonfile(HeaderLessonfile, TestSupport):
     Common lesson file class for some interval exercises.
     We inherit from TestSupport, but overwrites some methods from it.
     """
+    def get_test_num_questions(self):
+        count, t = parse_test_def(self.header.test)
+        if self.header.intervals:
+            return len(self.header.intervals) * count
+        else:
+            return len(self.header.ask_for_intervals_0) * count
     def enter_test_mode(self):
         count, t = parse_test_def(self.header.test)
         if self.header.intervals:
@@ -1471,27 +1521,166 @@ class ElembuilderLessonfile(QuestionsLessonfile):
             self.blocklists['element'] = []
 
 
+class InfoCache(object):
+    class InfoCacheException(IOError):
+        pass
+    class FileNotFound(InfoCacheException):
+        pass
+    class FileNotLessonfile(InfoCacheException):
+        pass
+    class FrontPageCache(object):
+        def __init__(self):
+            self._data = {}
+        def get(self, filename, field):
+            if filename not in self._data:
+                self.parse_file(filename)
+            elif os.path.getmtime(filename) != self._data[filename]['mtime']:
+                self.parse_file(filename)
+            return self._data[filename][field]
+        def parse_file(self, filename):
+            from solfege import learningtree
+            try:
+                p = learningtree.load_tree(filename)
+                self._data[filename] = {'title': p.m_name,
+                                        'mtime': os.path.getmtime(filename)}
+            except Exception, exception:
+                # FIXME this is a hack
+                self._data[filename] = {'title': str(exception).decode(sys.getfilesystemencoding(), 'replace'), 'mtime': None}
+    def __init__(self):
+        self._data = {}
+        self._dir_mtime = {}
+        self.frontpage = self.FrontPageCache()
+    def get(self, filename, field):
+        """
+        filename is either a solfege: uri or an absolute file name.
+        Relative file names are not allowed, since we want the program
+        to refer to standard lesson files with the solfege: uri
+        """
+        assert is_uri(filename) or os.path.isabs(filename)
+        if not os.path.isfile(uri_expand(filename)):
+            raise InfoCache.FileNotFound(filename)
+        mtime = os.path.getmtime(uri_expand(filename))
+        if filename not in self._data or self._data[filename]['mtime'] < mtime:
+            self.parse_file(filename, mtime)
+        return self._data[filename][field]
+    def parse_file(self, filename, mtime=None):
+        logging.debug("parse_file %s" % filename)
+        assert is_uri(filename) or os.path.isabs(filename)
+        p = parse_lesson_file_header(uri_expand(filename))
+        if not p:
+            raise self.FileNotLessonfile(filename)
+        self._data[filename] = {
+            'title': p.header.get('title', 'error: no title in file'),
+            'module': p.header['module'],
+            'test': p.header.get('test', None),
+            'test_requirement': p.header.get('test_requirement', None),
+            'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
+            'replaces': p.header.get('replaces', []),
+        }
+    def iter_parse_all_files(self):
+        """
+        Parse the files and put then in the cache.
+        yield filename.
+        """
+        logging.debug("iter_parse_all_files()")
+        for filename in self._iter_files(
+                os.path.join(exercises_dir, u"lesson-files")):
+            yield filename
+        for filename in self.iter_user_files():
+            yield filename
+    def _iter_files(self, *path):
+        """
+        Parse and put into the cache all lesson files in the directories
+        in the list path and yield the file names of the files that are
+        parsed and found to be lesson files.
+        """
+        for directory in path:
+            if os.path.isdir(directory):
+                for fn in os.listdir(directory):
+                    filename = os.path.join(directory, fn)
+                    if not os.path.isfile(filename):
+                        continue
+                    filename = mk_uri(filename)
+                    try:
+                        self.parse_file(filename)
+                    except self.FileNotLessonfile:
+                        continue
+                    yield filename
+    def parse_all_files(self, when_idle):
+        """
+        Parse all standard lesson files and the user_lessonfiles.
+        Will not check if reparse is necessary.
+        """
+        logging.debug("parse_all_files(when_idle=%s)" % when_idle)
+        if when_idle:
+            self._lessonfiles_iterator = self.iter_parse_all_files()
+            def on_idle_parse():
+                try:
+                    filename = self._lessonfiles_iterator.next()
+                    return True
+                except StopIteration:
+                    logging.debug("parse_all_files(...) done.")
+                    return False
+            gobject.idle_add(on_idle_parse)
+        else:
+            list(self.iter_parse_all_files())
+    def update_modified_files(self):
+        self.cond_parse_dir(filesystem.user_lessonfiles())
+        self.cond_parse_dir(os.path.join(exercises_dir, u"lesson-files"))
+    def cond_parse_dir(self, dir):
+        """
+        Check the mtime of the files in dir if dirs mtime has changed
+        or is not in _data. Parse the lesson files if their mtime has
+        changed or is not in _data.
+        Do nothing if the dir does not exist.
+        """
+        if not os.path.exists(dir):
+            return
+        mtime = os.path.getmtime(dir)
+        assert isinstance(dir, unicode)
+        logging.debug("cond_parse_dir(%s) mtime=%s" % (dir, mtime))
+        if dir not in self._dir_mtime:
+            for filename in os.listdir(dir):
+                fn = os.path.join(dir, filename)
+                if not os.path.isfile(fn):
+                    continue
+                fn = mk_uri(fn)
+                file_mtime = os.path.getmtime(uri_expand(fn))
+                if fn not in self._data or file_mtime > self._data[fn]['mtime']:
+                    try:
+                        logging.debug(" will parse %s. in data:%s" % (fn, fn in self._data))
+                        self.parse_file(fn, file_mtime)
+                    except self.InfoCacheException:
+                        logging.debug(" exception, not parsed: %s" % fn)
+                        pass
+    def iter_user_files(self):
+        """
+        Put the data from the lesson files below ~/.solfege/exercises/
+        in the cache and yield the filename.
+        """
+        logging.debug("iter_user_files()")
+        for fn in glob.glob(os.path.join(filesystem.user_data(), u"exercises/*/*/*")):
+            if os.path.isfile(fn):
+                try:
+                    self.parse_file(fn)
+                    yield fn
+                except self.InfoCacheException:
+                    continue
+
 
 class LessonFileManager:
+    class UnknownLessonID(Exception):
+        pass
     def __init__(self):
         # A list of info about lesson files that where discarded
         # by the lesson file manager
         self.m_discards = []
         self.m_uiddb = {}
-    def old_count(self):
-        """
-        Return the number of different lesson_ids we have in the
-        old statistics directory.
-        """
-        count = 0
-        for p in 'lesson-files', filesystem.user_lessonfiles():
-            count += len(glob.glob("%s/*" % p))
-        return count
     def parse(self, debug):
-        lessonpath = ['lesson-files',
+        lessonpath = [os.path.join('exercises', 'lesson-files'),
                       filesystem.user_lessonfiles()]
         if debug:
-            lessonpath.append('regression-lesson-files')
+            lessonpath.append(os.path.join('exercises', 'regression-lesson-files'))
         self.m_uiddb = {}
         for dir in lessonpath:
             if not os.path.isdir(dir):
@@ -1505,7 +1694,9 @@ class LessonFileManager:
                   or not os.path.isfile(filename):
                     continue
                 lesson_id = self.parse_into_uiddb(filename)
-                yield lesson_id, self.m_uiddb[lesson_id]
+                # parse_into_uiddb will return None on error.
+                if lesson_id:
+                    yield lesson_id, self.m_uiddb[lesson_id]
     def parse_into_uiddb(self, filename):
                 """
                 Returns the lesson_id of the parsed file.
@@ -1521,20 +1712,11 @@ class LessonFileManager:
                         'exception': e,
                     })
                     return
-                msg = 'Failed to parse the lessonfile "%s". Do not report this as a bug. Fix the file, or delete it.'
-                if os.path.isabs(filename):
-                    msg = msg % filename
-                else:
-                    msg = msg % (os.path.join(os.getcwdu(), filename))
-                msg = "\n".join(textwrap.wrap(msg, 110)) + "\n"
                 try:
                     p = parse_lesson_file_header(filename)
                 except dataparser.DataparserException, e:
-                    print >> sys.stderr, msg
-                    print >> sys.stderr, str(e)
                     return
                 if not p:
-                    print >> sys.stderr, msg
                     return
                 if not 'title' in p.header:
                     p.header['title'] = '@@@NOT TITLE'
@@ -1552,55 +1734,32 @@ class LessonFileManager:
                 if not p.header['lesson_id'] in self.m_uiddb:
                     self.m_uiddb[p.header['lesson_id']] = n
                 else:
-                    if type(self.m_uiddb[p.header['lesson_id']]) != list:
-                        self.m_uiddb[p.header['lesson_id']] = \
-                            [self.m_uiddb[p.header['lesson_id']]]
-                    self.m_uiddb[p.header['lesson_id']].append(n)
+                    # We do the simplest thing here, we keep the oldest
+                    # file and discard info from other files with the
+                    # same lesson_id. Users are in the release notes
+                    # for 3.15.3 advised to solve lesson_id conflict before
+                    # upgrading.
+                    if n['mtime'] < self.m_uiddb[p.header['lesson_id']]['mtime']:
+                        self.m_uiddb[p.header['lesson_id']] = n
                 return p.header['lesson_id']
     def get(self, lesson_id, fieldname):
+        if lesson_id not in self.m_uiddb:
+            raise self.UnknownLessonID(lesson_id)
         if fieldname in ('title', 'test', 'module', 'lesson_heading'):
             if fieldname in self.m_uiddb[lesson_id]['header']:
                 return self.m_uiddb[lesson_id]['header'][fieldname]
         if fieldname in self.m_uiddb[lesson_id]:
             return self.m_uiddb[lesson_id][fieldname]
-    def ignore_duplicates_with_lesson_id(self, lesson_id):
-        """
-        Discard the extra entries of lesson_id, keep the oldest file.
-        """
-        def sort_func(a, b):
-            return cmp(a['mtime'], b['mtime'])
-        self.m_uiddb[lesson_id].sort(sort_func)
-        self.m_uiddb[lesson_id] = self.m_uiddb[lesson_id][0]
-    def duplicate_id_keep_this(self, lesson_id, filename):
-        """
-        For the lesson_id conflicts with lesson_id, keep the lesson_id for
-        the file 'filename', add new lesson_id to the other files.
-        """
-        for info in self.m_uiddb[lesson_id]:
-            if info['filename'] != filename:
-                p = LessonIdParser(info['filename'])
-                p.new_lesson_id()
-                self.parse_into_uiddb(info['filename'])
-        self.m_uiddb[lesson_id] = [d for d in self.m_uiddb[lesson_id] if d['filename'] == filename][0]
-    def iterate_duplicated_lesson_id(self):
-        """
-        Return the a lesson_id if there exist lesson_ids that
-        are duplicated. If not, return None.
-        """
-        for v in self.m_uiddb.values():
-            if isinstance(v, list):
-                yield v[0]['header']['lesson_id']
-    def iterate_lesson_ids(self):
-        for k in self.m_uiddb:
-            yield k
-    def get_lesson_file_info(self, lesson_id):
-        """
-        FIXME: the function name is not very good.
-        Return data used to fix things when we have a lesson_id crash.
-        """
-        return [{'filename': d['filename'].decode(sys.getfilesystemencoding(), 'replace'),
-            'timestr': time.strftime('%c', time.localtime(d['mtime'])).decode(sys.getdefaultencoding(), 'replace'),
-            'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
+
+mgr = None
+
+def require_mgr():
+    global mgr
+    if not mgr:
+        mgr = LessonFileManager()
+        # This causes the LessonFileManager to parse all lesson files
+        list(mgr.parse(True))
+
 
 def parse_lesson_file_header(filename):
     """
@@ -1612,14 +1771,21 @@ def parse_lesson_file_header(filename):
     Return None if we find no header block.
     """
     r = re.compile("\\header\s*{.*?}", re.MULTILINE|re.DOTALL)
-    s = open(filename, 'rU').read()
+    # We cannot read the whole file, since we don't know what the user
+    # have placed in the directory. It could be a whole DVD iso image.
+    # The actual size we read, 40k, is mentioned in the user manual,
+    # and must be updated there too if changed.
+    s = open(filename, 'rU').read(40960)
     m = r.search(s)
     p = dataparser.Dataparser(predef, lessonfile_functions_no_i18n)
     p.m_location = os.path.split(filename)
     p.m_ignore_lookup_error = True
     if not m:
         return
-    p.parse_string(m.group())
+    try:
+        p.parse_string(m.group())
+    except dataparser.DataparserException:
+        return None
     return p
 
 
