@@ -468,7 +468,7 @@ class TestWavfile(unittest.TestCase):
     def setUp(self):
         self.p = QuestionsLessonfile()
         self.p.parse_string("""
-        question { music = wavfile("lesson-files/share/fifth-small-293.33.wav") }
+        question { music = wavfile("exercises/lesson-files/share/fifth-small-293.33.wav") }
         """)
         self.p._idx = 0
     def test_play(self):
@@ -479,16 +479,16 @@ class TestWavfile(unittest.TestCase):
     def test_get_mpd_music_string(self):
         question = self.p.m_questions[0]
         self.assertEquals(question['music'].get_mpd_music_string(self.p),
-                          "Wavfile:lesson-files/share/fifth-small-293.33.wav")
+                          "Wavfile:exercises/lesson-files/share/fifth-small-293.33.wav")
         self.assertEquals(question.music.m_musicdata,
-                          "lesson-files/share/fifth-small-293.33.wav")
+                          "exercises/lesson-files/share/fifth-small-293.33.wav")
         self.assertTrue(isinstance(question.music, Wavfile))
 
 class TestMidifile(unittest.TestCase):
     def setUp(self):
         self.p = QuestionsLessonfile()
         self.p.parse_string("""
-        question { music = midifile("lesson-files/share/fanfare.midi") }
+        question { music = midifile("exercises/lesson-files/share/fanfare.midi") }
         """)
         self.p._idx = 0
     def test_play(self):
@@ -499,9 +499,9 @@ class TestMidifile(unittest.TestCase):
     def test_get_mpd_music_string(self):
         question = self.p.m_questions[0]
         self.assertEquals(question['music'].get_mpd_music_string(self.p),
-                          "Midifile:lesson-files/share/fanfare.midi")
+                          "Midifile:exercises/lesson-files/share/fanfare.midi")
         self.assertEquals(question.music.m_musicdata,
-                          "lesson-files/share/fanfare.midi")
+                          "exercises/lesson-files/share/fanfare.midi")
         self.assertTrue(isinstance(question.music, Midifile))
 
 
@@ -509,7 +509,7 @@ class TestCmdline(unittest.TestCase):
     def setUp(self):
         self.p = QuestionsLessonfile()
         self.p.parse_string("""
-        question { music = cmdline("/usr/bin/play lesson-files/share/fifth-pure-220.00.wav") }
+        question { music = cmdline("/usr/bin/play exercises/lesson-files/share/fifth-pure-220.00.wav") }
         """)
         self.p._idx = 0
     def test_play(self):
@@ -540,7 +540,7 @@ f1     0    4096 10   1  ; use GEN10 to compute a sine wave
   i1  +     1   10000  660   0.08    0.08
 e
  """
- music = csound(load("lesson-files/share/sinus-ad.orc"), csound_sco)
+ music = csound(load("exercises/lesson-files/share/sinus-ad.orc"), csound_sco)
 }''')
         question = self.p.m_questions[0]
         if cfg.get_bool('testing/may_play_sound'):
